@@ -1,16 +1,18 @@
 /*
- * PCBIT-D module support
- *
  * Copyright (C) 1996 Universidade de Lisboa
  * 
  * Written by Pedro Roque Marques (roque@di.fc.ul.pt)
  *
  * This software may be used and distributed according to the terms of 
- * the GNU General Public License, incorporated herein by reference.
+ * the GNU Public License, incorporated herein by reference.
+ */
+
+/*        
+ *        PCBIT-D module support
  */
 
 #include <linux/module.h>
-#include <linux/init.h>
+
 #include <linux/sched.h>
 #include <linux/string.h>
 #include <linux/kernel.h>
@@ -20,29 +22,32 @@
 #include <linux/isdnif.h>
 #include "pcbit.h"
 
-MODULE_DESCRIPTION("ISDN4Linux: Driver for PCBIT-T card");
-MODULE_AUTHOR("Pedro Roque Marques");
-MODULE_LICENSE("GPL");
-MODULE_PARM(mem, "1-" __MODULE_STRING(MAX_PCBIT_CARDS) "i");
-MODULE_PARM(irq, "1-" __MODULE_STRING(MAX_PCBIT_CARDS) "i");
-
 static int mem[MAX_PCBIT_CARDS] = {0, };
 static int irq[MAX_PCBIT_CARDS] = {0, };
 
 static int num_boards;
-struct pcbit_dev * dev_pcbit[MAX_PCBIT_CARDS] = {0, };
+struct pcbit_dev * dev_pcbit[MAX_PCBIT_CARDS] = {0, 0, 0, 0};
+
+int init_module(void);
+void cleanup_module(void);
 
 extern void pcbit_terminate(int board);
 extern int pcbit_init_dev(int board, int mem_base, int irq);
 
-static int __init pcbit_init(void)
+#ifdef MODULE
+MODULE_PARM(mem, "1-" __MODULE_STRING(MAX_PCBIT_CARDS) "i");
+MODULE_PARM(irq, "1-" __MODULE_STRING(MAX_PCBIT_CARDS) "i");
+#define pcbit_init init_module
+#endif
+
+int pcbit_init(void)
 {
 	int board;
 
 	num_boards = 0;
 
-	printk(KERN_NOTICE 
-	       "PCBIT-D device driver v 0.5-fjpc0 19991204 - "
+	printk(KERN_INFO 
+	       "PCBIT-D device driver v 0.5 - "
 	       "Copyright (C) 1996 Universidade de Lisboa\n");
 
 	if (mem[0] || irq[0]) 
@@ -78,23 +83,40 @@ static int __init pcbit_init(void)
 		else
 			return -EIO;
 	}
+
+	/* No symbols to export, hide all symbols */
+	EXPORT_NO_SYMBOLS;
+
 	return 0;
 }
 
-static void  pcbit_exit(void)
+#ifdef MODULE
+void cleanup_module(void)
 {
 	int board;
 
 	for (board = 0; board < num_boards; board++)
 		pcbit_terminate(board);
-	printk(KERN_NOTICE 
+	printk(KERN_INFO 
 	       "PCBIT-D module unloaded\n");
 }
 
-#ifndef MODULE
+#else
+#ifdef COMPAT_HAS_NEW_SETUP
+#define MAX_PARA	(MAX_PCBIT_CARDS * 2)
+#include <linux/init.h>
+static int __init pcbit_setup(char *line)
+{
+	int i, j, argc;
+	char *str;
+	int ints[MAX_PARA+1];
+
+	str = get_options(line, MAX_PARA, ints);
+#else
 void pcbit_setup(char *str, int *ints)
 {
 	int i, j, argc;
+#endif
 	argc = ints[0];
 	i = 0;
 	j = 1;
@@ -113,9 +135,14 @@ void pcbit_setup(char *str, int *ints)
 
 		i++;
 	}
+#ifdef COMPAT_HAS_NEW_SETUP
+	return(1);
+}
+__setup("pcbit=", pcbit_setup);
+#else
 }
 #endif
+#endif
+
 
-module_init(pcbit_init);
-module_exit(pcbit_exit);
 
