@@ -50,15 +50,30 @@
 
 #define USB_REQ_CPIA_GRAB_FRAME			0xC1
 #define USB_REQ_CPIA_UPLOAD_FRAME		0xC2
+#define  WAIT_FOR_NEXT_FRAME			0
+#define  FORCE_FRAME_UPLOAD			1
 #define USB_REQ_CPIA_SET_GRAB_MODE		0xC3
 #define USB_REQ_CPIA_INIT_STREAM_CAP		0xC4
 #define USB_REQ_CPIA_FINI_STREAM_CAP		0xC5
 #define USB_REQ_CPIA_START_STREAM_CAP		0xC6
 #define USB_REQ_CPIA_END_STREAM_CAP		0xC7
 #define USB_REQ_CPIA_SET_FORMAT			0xC8
+#define  FORMAT_QCIF	0
+#define  FORMAT_CIF	1
+#define  FORMAT_YUYV	0
+#define  FORMAT_UYVY	1
+#define  FORMAT_420	0
+#define  FORMAT_422	1
 #define USB_REQ_CPIA_SET_ROI			0xC9
 #define USB_REQ_CPIA_SET_COMPRESSION		0xCA
+#define  COMP_DISABLED	0
+#define  COMP_AUTO	1
+#define  COMP_MANUAL	2
+#define  DONT_DECIMATE	0
+#define  DECIMATE	1
 #define USB_REQ_CPIA_SET_COMPRESSION_TARGET	0xCB
+#define  TARGET_QUALITY		0
+#define  TARGET_FRAMERATE	1
 #define USB_REQ_CPIA_SET_YUV_THRESH		0xCC
 #define USB_REQ_CPIA_SET_COMPRESSION_PARAMS	0xCD
 #define USB_REQ_CPIA_DISCARD_FRAME		0xCE
@@ -72,12 +87,6 @@
 #define USB_REQ_CPIA_DOWNLOAD_DRAM		0xEA
 /* #define USB_REQ_CPIA_NULL_CMD		0x?? */
 
-#define CPIA_QCIF	0
-#define CPIA_CIF	1
-
-#define CPIA_YUYV	0
-#define CPIA_UYVY	1
-
 #define STREAM_BUF_SIZE		(PAGE_SIZE * 4)
 /* #define STREAM_BUF_SIZE	(FRAMES_PER_DESC * FRAME_SIZE_PER_DESC) */
 
