@@ -1,3 +1,11 @@
+/*
+	Fixes:
+	1999/09/04 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+		Handle states in usb_kbd_irq
+	1999/09/06 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+		rmmod usb-keyboard doesn't crash the system anymore: the irq
+		handlers are correctly released with usb_release_irq
+*/
 #include <linux/kernel.h>
 #include <linux/malloc.h>
 #include <linux/string.h>
@@ -13,7 +21,7 @@
 #define PCKBD_RELEASED 0x80
 #define PCKBD_NEEDS_E0 0x80
 
-#define USBKBD_MODIFIER_BASE  120
+#define USBKBD_MODIFIER_BASE  224
 #define USBKBD_KEYCODE_OFFSET 2
 #define USBKBD_KEYCODE_COUNT  6
 
@@ -31,12 +39,14 @@ struct usb_keyboard
     unsigned char repeat_key;
     struct timer_list repeat_timer;
     struct list_head list;
+    unsigned int irqpipe;
+    void *irq_handler; /* host controller's IRQ transfer handle */
 };
 
 extern unsigned char usb_kbd_map[];
 
-static int usb_kbd_probe(struct usb_device *dev);
-static void usb_kbd_disconnect(struct usb_device *dev);
+static void * usb_kbd_probe(struct usb_device *dev, unsigned int i);
+static void usb_kbd_disconnect(struct usb_device *dev, void *ptr);
 static void usb_kbd_repeat(unsigned long dummy);
 
 static LIST_HEAD(usb_kbd_list);
@@ -97,8 +107,22 @@ usb_kbd_irq(int state, void *buffer, int len, void *dev_id)
     struct usb_keyboard *kbd = (struct usb_keyboard*) dev_id;
     unsigned long *down = (unsigned long*) buffer;
 
-    if(kbd->down[0] != down[0] || kbd->down[1] != down[1])
-    {
+    /*
+     * USB_ST_NOERROR is the normal case.
+     * USB_ST_REMOVED occurs if keyboard disconnected
+     * On other states, ignore
+     */
+
+    switch (state) {
+	case USB_ST_REMOVED:
+	case USB_ST_INTERNALERROR:
+        	printk(KERN_DEBUG "%s(%d): Suspending\n", __FILE__, __LINE__);
+		return 0; /* disable */
+	case USB_ST_NOERROR: break;
+		default: return 1; /* ignore */
+    }
+
+    if(kbd->down[0] != down[0] || kbd->down[1] != down[1]) {
         unsigned char *olddown, *newdown;
         unsigned char modsdelta, key;
         int i;
@@ -166,24 +190,22 @@ usb_kbd_irq(int state, void *buffer, int len, void *dev_id)
     return 1;
 }
 
-static int
-usb_kbd_probe(struct usb_device *dev)
+static void *
+usb_kbd_probe(struct usb_device *dev, unsigned int i)
 {
     struct usb_interface_descriptor *interface;
     struct usb_endpoint_descriptor *endpoint;
     struct usb_keyboard *kbd;
-
-    if (dev->descriptor.bNumConfigurations < 1)
-	return -1;
-
-    interface = &dev->config[0].altsetting[0].interface[0];
+    int ret;
+    
+    interface = &dev->actconfig->interface[i].altsetting[0];
     endpoint = &interface->endpoint[0];
 
     if(interface->bInterfaceClass != 3
        || interface->bInterfaceSubClass != 1
        || interface->bInterfaceProtocol != 1)
     {
-        return -1;
+        return NULL;
     }
 
     printk(KERN_INFO "USB HID boot protocol keyboard detected.\n");
@@ -193,42 +215,40 @@ usb_kbd_probe(struct usb_device *dev)
     {
         memset(kbd, 0, sizeof(*kbd));
         kbd->dev = dev;
-        dev->private = kbd;
 
-        if (usb_set_configuration(dev, dev->config[0].bConfigurationValue)) {
-		printk (KERN_INFO " Failed usb_set_configuration: kbd\n");
-		goto probe_err;
-	}
         usb_set_protocol(dev, 0);
         usb_set_idle(dev, 0, 0);
         
-        usb_request_irq(dev,
-                        usb_rcvctrlpipe(dev, endpoint->bEndpointAddress),
-                        usb_kbd_irq,
-                        endpoint->bInterval,
-                        kbd);
+	kbd->irqpipe = usb_rcvctrlpipe(dev, endpoint->bEndpointAddress);
+	ret = usb_request_irq(dev, kbd->irqpipe,
+				usb_kbd_irq, endpoint->bInterval,
+				kbd, &kbd->irq_handler);
+	if (ret) {
+		printk(KERN_INFO "usb-keyboard failed usb_request_irq (0x%x)\n", ret);
+		goto probe_err;
+	}
 
         list_add(&kbd->list, &usb_kbd_list);
 	
-	return 0;
+	return kbd;
     }
 
 probe_err:
     if (kbd)
     	kfree (kbd);
-    return -1;
+    return NULL;
 }
 
 static void
-usb_kbd_disconnect(struct usb_device *dev)
+usb_kbd_disconnect(struct usb_device *dev, void *ptr)
 {
-    struct usb_keyboard *kbd = (struct usb_keyboard*) dev->private;
-    if(kbd)
+    struct usb_keyboard *kbd = (struct usb_keyboard*) ptr;
+    if (kbd)
     {
-        dev->private = NULL;
-        list_del(&kbd->list);
-        del_timer(&kbd->repeat_timer);
-        kfree(kbd);
+	usb_release_irq(dev, kbd->irq_handler, kbd->irqpipe);
+	list_del(&kbd->list);
+	del_timer(&kbd->repeat_timer);
+	kfree(kbd);
     }
 
     printk(KERN_INFO "USB HID boot protocol keyboard removed.\n");
@@ -240,6 +260,30 @@ int usb_kbd_init(void)
     return 0;
 }
 
+void usb_kbd_cleanup(void)
+{
+	struct list_head *cur, *head = &usb_kbd_list;
+
+	cur = head->next;
+
+	while (cur != head) {
+		struct usb_keyboard *kbd = list_entry(cur, struct usb_keyboard, list);
+
+		cur = cur->next;
+
+                list_del(&kbd->list);
+                INIT_LIST_HEAD(&kbd->list);
+
+		if (kbd->irq_handler) {
+			usb_release_irq(kbd->dev, kbd->irq_handler, kbd->irqpipe);
+			/* never keep a reference to a released IRQ! */
+			kbd->irq_handler = NULL;
+		}
+	}
+
+	usb_deregister(&usb_kbd_driver);
+}
+
 #ifdef MODULE
 int init_module(void)
 {
@@ -248,7 +292,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-	usb_deregister(&usb_kbd_driver);
+	usb_kbd_cleanup();
 }
 #endif
 
