@@ -34,19 +34,15 @@
 #include <linux/sched.h>
 #include <linux/signal.h>
 #include <linux/errno.h>
-#include <linux/miscdevice.h>
 #include <linux/random.h>
 #include <linux/poll.h>
 #include <linux/init.h>
 #include <linux/malloc.h>
 #include <linux/module.h>
-
-#include <asm/spinlock.h>
+#include <linux/spinlock.h>
 
 #include "usb.h"
 
-#define USB_MOUSE_MINOR 32
-
 struct mouse_state {
 	unsigned char buttons; /* current button state */
 	long dx; /* dx, dy, dz are change since last read */
@@ -110,7 +106,7 @@ static int mouse_irq(int state, void *__buffer, int len, void *dev_id)
 
 	/* if the USB mouse sends an interrupt, then something noteworthy
 	   must have happened */
-	mouse->buttons = data[0] & 0x07;
+	mouse->buttons = data[0] & 0x0f;
 	mouse->dx += data[1]; /* data[] is signed, so this works */
 	mouse->dy -= data[2]; /* y-axis is reversed */
 	mouse->dz -= data[3];
@@ -121,7 +117,7 @@ static int mouse_irq(int state, void *__buffer, int len, void *dev_id)
 
 	wake_up_interruptible(&mouse->wait);
 	if (mouse->fasync)
-		kill_fasync(mouse->fasync, SIGIO);
+		kill_fasync(mouse->fasync, SIGIO, POLL_IN);
 
 	return 1;
 }
@@ -149,7 +145,8 @@ static int release_mouse(struct inode * inode, struct file * file)
 	if (--mouse->active == 0) {
 		mouse->suspended = 0;
 		/* stop polling the mouse while its not in use */
-	    	usb_release_irq(mouse->dev, mouse->irq_handle);
+	    	usb_release_irq(mouse->dev, mouse->irq_handle,
+				usb_rcvctrlpipe(mouse->dev, mouse->bEndpointAddress));
 		/* never keep a reference to a released IRQ! */
 		mouse->irq_handle = NULL;
 	}
@@ -160,6 +157,7 @@ static int release_mouse(struct inode * inode, struct file * file)
 static int open_mouse(struct inode * inode, struct file * file)
 {
 	struct mouse_state *mouse = &static_mouse_state;
+	int ret;
 
 	printk(KERN_DEBUG "%s(%d): open_mouse\n", __FILE__, __LINE__);
 	/*
@@ -195,7 +193,13 @@ static int open_mouse(struct inode * inode, struct file * file)
 		return 0;
 
 	/* start the usb controller's polling of the mouse */
-	mouse->irq_handle = usb_request_irq(mouse->dev, usb_rcvctrlpipe(mouse->dev, mouse->bEndpointAddress), mouse_irq, mouse->bInterval, NULL);
+	ret = usb_request_irq(mouse->dev, usb_rcvctrlpipe(mouse->dev, mouse->bEndpointAddress),
+			mouse_irq, mouse->bInterval,
+			NULL, &mouse->irq_handle);
+	if (ret) {
+		printk (KERN_WARNING "usb-mouse: usb_request_irq failed (0x%x)\n", ret);
+		return ret;
+	}
 
 	return 0;
 }
@@ -312,54 +316,38 @@ struct file_operations usb_mouse_fops = {
 	fasync_mouse,
 };
 
-static struct miscdevice usb_mouse = {
-	USB_MOUSE_MINOR, "USB mouse", &usb_mouse_fops
-};
-
-static int mouse_probe(struct usb_device *dev)
+static void* mouse_probe(struct usb_device *dev, unsigned int i)
 {
 	struct usb_interface_descriptor *interface;
 	struct usb_endpoint_descriptor *endpoint;
 	struct mouse_state *mouse = &static_mouse_state;
-
-	/* We don't handle multi-config mice */
-	if (dev->descriptor.bNumConfigurations != 1)
-		return -1;
-
-	/* We don't handle multi-interface mice */
-	if (dev->config[0].bNumInterfaces != 1)
-		return -1;
+	int ret;
 
 	/* Is it a mouse interface? */
-	interface = &dev->config[0].interface[0].altsetting[0];
+	interface = &dev->actconfig->interface[i].altsetting[0];
 	if (interface->bInterfaceClass != 3)
-		return -1;
+		return NULL;
 	if (interface->bInterfaceSubClass != 1)
-		return -1;
+		return NULL;
 	if (interface->bInterfaceProtocol != 2)
-		return -1;
+		return NULL;
 
 	/* Multiple endpoints? What kind of mutant ninja-mouse is this? */
 	if (interface->bNumEndpoints != 1)
-		return -1;
+		return NULL;
 
 	endpoint = &interface->endpoint[0];
 
 	/* Output endpoint? Curiousier and curiousier.. */
 	if (!(endpoint->bEndpointAddress & 0x80))
-		return -1;
+		return NULL;
 
 	/* If it's not an interrupt endpoint, we'd better punt! */
 	if ((endpoint->bmAttributes & 3) != 3)
-		return -1;
+		return NULL;
 
 	printk("USB mouse found\n");
 
-	if (usb_set_configuration(dev, dev->config[0].bConfigurationValue)) {
-		printk (KERN_INFO " Failed usb_set_configuration: mouse\n");
-		return -1;
-	}
-
 	/* these are used to request the irq when the mouse is opened */
 	mouse->dev = dev;
 	mouse->bEndpointAddress = endpoint->bEndpointAddress;
@@ -373,22 +361,27 @@ static int mouse_probe(struct usb_device *dev)
 	{
 		printk(KERN_DEBUG "%s(%d): mouse resume\n", __FILE__, __LINE__);
 		/* restart the usb controller's polling of the mouse */
-		mouse->irq_handle = usb_request_irq(mouse->dev,
-			usb_rcvctrlpipe(mouse->dev, mouse->bEndpointAddress),
-			mouse_irq, mouse->bInterval, NULL);
+		ret = usb_request_irq(mouse->dev, usb_rcvctrlpipe(mouse->dev, mouse->bEndpointAddress),
+			mouse_irq, mouse->bInterval,
+			NULL, &mouse->irq_handle);
+		if (ret) {
+			printk (KERN_WARNING "usb-mouse: usb_request_irq failed (0x%x)\n", ret);
+			return NULL;
+		}
 		mouse->suspended = 0;
 	}
 
-	return 0;
+	return mouse;
 }
 
-static void mouse_disconnect(struct usb_device *dev)
+static void mouse_disconnect(struct usb_device *dev, void *ptr)
 {
-	struct mouse_state *mouse = &static_mouse_state;
+	struct mouse_state *mouse = ptr;
 
 	/* stop the usb interrupt transfer */
 	if (mouse->present) {
-	    	usb_release_irq(mouse->dev, mouse->irq_handle);
+	    	usb_release_irq(mouse->dev, mouse->irq_handle,
+				usb_rcvctrlpipe(mouse->dev, mouse->bEndpointAddress));
 		/* never keep a reference to a released IRQ! */
 	}
 
@@ -396,14 +389,16 @@ static void mouse_disconnect(struct usb_device *dev)
 
 	/* this might need work */
 	mouse->present = 0;
-	printk("Mouse disconnected\n");
+	printk("USB Mouse disconnected\n");
 }
 
 static struct usb_driver mouse_driver = {
 	"mouse",
 	mouse_probe,
 	mouse_disconnect,
-	{ NULL, NULL }
+	{ NULL, NULL },
+	&usb_mouse_fops,
+	16
 };
 
 int usb_mouse_init(void)
@@ -415,8 +410,6 @@ int usb_mouse_init(void)
 	init_waitqueue_head(&mouse->wait);
 	mouse->fasync = NULL;
 
-	misc_register(&usb_mouse);
-
 	usb_register(&mouse_driver);
 	printk(KERN_INFO "USB HID boot protocol mouse driver registered.\n");
 	return 0;
@@ -428,14 +421,14 @@ void usb_mouse_cleanup(void)
 
 	/* stop the usb interrupt transfer */
 	if (mouse->present) {
-	    	usb_release_irq(mouse->dev, mouse->irq_handle);
+	    	usb_release_irq(mouse->dev, mouse->irq_handle,
+				usb_rcvctrlpipe(mouse->dev, mouse->bEndpointAddress));
 		/* never keep a reference to a released IRQ! */
 		mouse->irq_handle = NULL;
 	}
 
 	/* this, too, probably needs work */
 	usb_deregister(&mouse_driver);
-	misc_deregister(&usb_mouse);
 }
 
 #ifdef MODULE
