@@ -31,6 +31,7 @@
  *                  ECP currently untested
  *   0.3  10.08.99  fixing merge errors
  *   0.4  13.08.99  Added Vendor/Product ID of Brad Hard's cable
+ *   0.5  20.09.99  usb_control_msg wrapper used
  *
  */
 
@@ -61,17 +62,11 @@ static int get_1284_register(struct parport *pp, unsigned char reg, unsigned cha
 	static const unsigned char regindex[9] = {
 		4, 0, 1, 5, 5, 0, 2, 3, 6
 	};
-        devrequest dr;
         int ret;
 
 	if (!usbdev)
 		return -1;
-        dr.requesttype = 0xc0;
-        dr.request = 3;
-        dr.value = ((unsigned int)reg) << 8;
-        dr.index = 0;
-        dr.length = 7;
-        ret = usbdev->bus->op->control_msg(usbdev, usb_rcvctrlpipe(usbdev,0), &dr, priv->reg, 7);
+        ret = usb_control_msg(usbdev, usb_rcvctrlpipe(usbdev,0), 3, 0xc0, ((unsigned int)reg) << 8, 0, priv->reg, 7, HZ);
         if (ret) {
                 printk(KERN_DEBUG "uss720: get_1284_register(%d) failed, status 0x%x\n",
 		       (unsigned int)reg, ret);
@@ -95,17 +90,11 @@ static int set_1284_register(struct parport *pp, unsigned char reg, unsigned cha
 {
 	struct parport_uss720_private *priv = pp->private_data;
 	struct usb_device *usbdev = priv->usbdev;
-        devrequest dr;
         int ret;
 
 	if (!usbdev)
 		return -1;
-        dr.requesttype = 0x40;
-        dr.request = 4;
-        dr.value = (((unsigned int)reg) << 8) | val;
-        dr.index = 0;
-        dr.length = 0;
-        ret = usbdev->bus->op->control_msg(usbdev, usb_sndctrlpipe(usbdev,0), &dr, NULL, 0);
+        ret = usb_control_msg(usbdev, usb_sndctrlpipe(usbdev,0), 4, 0x40, (((unsigned int)reg) << 8) | val, 0, NULL, 0, HZ);
         if (ret) {
                 printk(KERN_DEBUG "uss720: set_1284_register(%u,0x%02x) failed, status 0x%x\n", 
 		       (unsigned int)reg, (unsigned int)val, ret);
@@ -374,7 +363,7 @@ static size_t parport_uss720_epp_write_data(struct parport *pp, const void *buf,
 		return 0;
 	if (change_mode(pp, ECR_EPP))
 		return 0;
-	i = usbdev->bus->op->bulk_msg(usbdev, usb_sndbulkpipe(usbdev, 1), buf, length, &rlen);
+	i = usbdev->bus->op->bulk_msg(usbdev, usb_sndbulkpipe(usbdev, 1), buf, length, &rlen, HZ*20);
 	if (i)
 		printk(KERN_ERR "uss720: sendbulk ep 1 buf %p len %u rlen %lu\n", buf, length, rlen);
 	change_mode(pp, ECR_PS2);
@@ -435,7 +424,7 @@ static size_t parport_uss720_ecp_write_data(struct parport *pp, const void *buff
 		return 0;
 	if (change_mode(pp, ECR_ECP))
 		return 0;
-	i = usbdev->bus->op->bulk_msg(usbdev, usb_sndbulkpipe(usbdev, 1), buffer, len, &rlen);
+	i = usbdev->bus->op->bulk_msg(usbdev, usb_sndbulkpipe(usbdev, 1), buffer, len, &rlen, HZ*20);
 	if (i)
 		printk(KERN_ERR "uss720: sendbulk ep 1 buf %p len %u rlen %lu\n", buffer, len, rlen);
 	change_mode(pp, ECR_PS2);
@@ -453,7 +442,7 @@ static size_t parport_uss720_ecp_read_data(struct parport *pp, void *buffer, siz
 		return 0;
 	if (change_mode(pp, ECR_ECP))
 		return 0;
-	i = usbdev->bus->op->bulk_msg(usbdev, usb_rcvbulkpipe(usbdev, 2), buffer, len, &rlen);
+	i = usbdev->bus->op->bulk_msg(usbdev, usb_rcvbulkpipe(usbdev, 2), buffer, len, &rlen, HZ*20);
 	if (i)
 		printk(KERN_ERR "uss720: recvbulk ep 2 buf %p len %u rlen %lu\n", buffer, len, rlen);
 	change_mode(pp, ECR_PS2);
@@ -486,7 +475,7 @@ static size_t parport_uss720_write_compat(struct parport *pp, const void *buffer
 		return 0;
 	if (change_mode(pp, ECR_PPF))
 		return 0;
-	i = usbdev->bus->op->bulk_msg(usbdev, usb_sndbulkpipe(usbdev, 1), buffer, len, &rlen);
+	i = usbdev->bus->op->bulk_msg(usbdev, usb_sndbulkpipe(usbdev, 1), buffer, len, &rlen, HZ*20);
 	if (i)
 		printk(KERN_ERR "uss720: sendbulk ep 1 buf %p len %u rlen %lu\n", buffer, len, rlen);
 	change_mode(pp, ECR_PS2);
@@ -549,7 +538,7 @@ static struct parport_operations parport_uss720_ops =
 
 /* --------------------------------------------------------------------- */
 
-static int uss720_probe(struct usb_device *usbdev)
+static void * uss720_probe(struct usb_device *usbdev, unsigned int ifnum)
 {
         struct usb_interface_descriptor *interface;
         struct usb_endpoint_descriptor *endpoint;
@@ -563,31 +552,16 @@ static int uss720_probe(struct usb_device *usbdev)
         if ((usbdev->descriptor.idVendor != 0x047e || usbdev->descriptor.idProduct != 0x1001) &&
 	    (usbdev->descriptor.idVendor != 0x0557 || usbdev->descriptor.idProduct != 0x2001) &&
 	    (usbdev->descriptor.idVendor != 0x0729 || usbdev->descriptor.idProduct != 0x1284))
-                return -1;
+                return NULL;
 
-        /* We don't handle multiple configurations */
-        if (usbdev->descriptor.bNumConfigurations != 1)
-                return -1;
+        /* our known interfaces have 3 alternate settings */
+        if (usbdev->actconfig->interface[ifnum].num_altsetting != 3)
+                return NULL;
 
-        /* We don't handle multiple interfaces */
-        if (usbdev->config[0].bNumInterfaces != 1)
-                return -1;
-
-        /* We don't handle multiple interfaces */
-        if (usbdev->config[0].interface[0].num_altsetting != 3)
-                return -1;
-
-        printk(KERN_DEBUG "uss720: set configuration\n");
-        usb_set_configuration(usbdev, usbdev->config[0].bConfigurationValue);
-
-        i = usb_set_interface(usbdev, 0, 2);
+        i = usb_set_interface(usbdev, ifnum, 2);
         printk(KERN_DEBUG "uss720: set inteface result %d\n", i);
 
-        interface = &usbdev->config[0].interface[0].altsetting[2];
-
-        //printk(KERN_DEBUG "uss720: get interface\n");
-        //i = usb_get_interface(usbdev, 0);
-        //printk(KERN_DEBUG "uss720: is in alternate setting %d\n", i);
+        interface = &usbdev->actconfig->interface[ifnum].altsetting[2];
 
 	/*
 	 * Allocate parport interface 
@@ -595,13 +569,13 @@ static int uss720_probe(struct usb_device *usbdev)
 	printk(KERN_INFO "uss720: (C) 1999 by Thomas Sailer, <sailer@ife.ee.ethz.ch>\n");
 
 	if (!(priv = kmalloc(sizeof(struct parport_uss720_private), GFP_KERNEL)))
-		return -1;
+		return NULL;
 	if (!(pp = parport_register_port(0, PARPORT_IRQ_NONE, PARPORT_DMA_NONE, &parport_uss720_ops))) {
-		kfree(priv);
-		return -1;
+		printk(KERN_WARNING "usb-uss720: could not register parport\n");
+		goto probe_abort;
 	}
+
 	pp->private_data = priv;
-        usbdev->private = pp;
 	priv->usbdev = usbdev;
 	pp->modes = PARPORT_MODE_PCSPP | PARPORT_MODE_TRISTATE | PARPORT_MODE_EPP | PARPORT_MODE_ECP | PARPORT_MODE_COMPAT;
 
@@ -623,26 +597,28 @@ static int uss720_probe(struct usb_device *usbdev)
 				  pp, &priv->irqhandle);
 	if (i) {
 		printk (KERN_WARNING "usb-uss720: usb_request_irq failed (0x%x)\n", i);
-		/* FIXME: undo some stuff and free some memory. */
-		return -1;
+		goto probe_abort_port;
 	}
 #endif
         parport_proc_register(pp);
         parport_announce_port(pp);
 
 	MOD_INC_USE_COUNT;
-        return 0;
+        return pp;
+
+probe_abort_port:
+	parport_unregister_port(pp);
+probe_abort:
+	kfree(priv);
+	return NULL;
 }
 
-static void uss720_disconnect(struct usb_device *usbdev)
+static void uss720_disconnect(struct usb_device *usbdev, void *ptr)
 {
-	struct parport *pp = (struct parport *)usbdev->private;
+	struct parport *pp = (struct parport *)ptr;
 	struct parport_uss720_private *priv = pp->private_data;
 
-#if 0
 	usb_release_irq(usbdev, priv->irqhandle, priv->irqpipe);
-#endif
-        usbdev->private = NULL;
         priv->usbdev = NULL;
 	parport_proc_unregister(pp);
 	parport_unregister_port(pp);
