@@ -6,9 +6,7 @@
  *
  */
 
-#include <linux/config.h>
 #include <linux/module.h>
-
 #include <linux/sched.h>
 #include <linux/kernel.h>
 #include <linux/mm.h>
@@ -77,6 +75,8 @@ smb_iget(struct super_block *sb, struct smb_fattr *fattr)
 	pr_debug("smb_iget: %p\n", fattr);
 
 	result = get_empty_inode();
+	if (!result)
+		return result;
 	result->i_sb = sb;
 	result->i_dev = sb->s_dev;
 	result->i_ino = fattr->f_ino;
@@ -577,7 +577,7 @@ static struct file_system_type smb_fs_type = {
 	NULL
 };
 
-__initfunc(int init_smb_fs(void))
+int __init init_smb_fs(void)
 {
 	return register_filesystem(&smb_fs_type);
 }
