@@ -8,8 +8,6 @@
 #define _ROSE_H 
 #include <linux/rose.h>
 
-#define	ROSE_SLOWHZ			10	/* Run timing at 1/10 second */
-
 #define	ROSE_ADDR_LEN			5
 
 #define	ROSE_MIN_LEN			3
@@ -45,22 +43,23 @@ enum {
 	ROSE_STATE_1,			/* Awaiting Call Accepted */
 	ROSE_STATE_2,			/* Awaiting Clear Confirmation */
 	ROSE_STATE_3,			/* Data Transfer */
-	ROSE_STATE_4			/* Awaiting Reset Confirmation */
+	ROSE_STATE_4,			/* Awaiting Reset Confirmation */
+	ROSE_STATE_5			/* Deferred Call Acceptance */
 };
 
-#define ROSE_DEFAULT_T0			(180 * ROSE_SLOWHZ)	/* Default T10 T20 value */
-#define ROSE_DEFAULT_T1			(200 * ROSE_SLOWHZ)	/* Default T11 T21 value */
-#define ROSE_DEFAULT_T2			(180 * ROSE_SLOWHZ)	/* Default T12 T22 value */
-#define	ROSE_DEFAULT_T3			(180 * ROSE_SLOWHZ)	/* Default T13 T23 value */
-#define	ROSE_DEFAULT_HB			(5 * ROSE_SLOWHZ)	/* Default Holdback value */
-#define	ROSE_DEFAULT_IDLE		(20 * 60 * ROSE_SLOWHZ)	/* Default No Activity value */
-#define	ROSE_DEFAULT_ROUTING		1			/* Default routing flag */
-#define	ROSE_DEFAULT_FAIL_TIMEOUT	(120 * ROSE_SLOWHZ)	/* Time until link considered usable */
-#define	ROSE_DEFAULT_MAXVC		50			/* Maximum number of VCs per neighbour */
-#define	ROSE_DEFAULT_WINDOW_SIZE	3			/* Default window size */
+#define ROSE_DEFAULT_T0			(180 * HZ)	/* Default T10 T20 value */
+#define ROSE_DEFAULT_T1			(200 * HZ)	/* Default T11 T21 value */
+#define ROSE_DEFAULT_T2			(180 * HZ)	/* Default T12 T22 value */
+#define	ROSE_DEFAULT_T3			(180 * HZ)	/* Default T13 T23 value */
+#define	ROSE_DEFAULT_HB			(5 * HZ)	/* Default Holdback value */
+#define	ROSE_DEFAULT_IDLE		(0 * 60 * HZ)	/* No Activity Timeout - none */
+#define	ROSE_DEFAULT_ROUTING		1		/* Default routing flag */
+#define	ROSE_DEFAULT_FAIL_TIMEOUT	(120 * HZ)	/* Time until link considered usable */
+#define	ROSE_DEFAULT_MAXVC		50		/* Maximum number of VCs per neighbour */
+#define	ROSE_DEFAULT_WINDOW_SIZE	7		/* Default window size */
 
 #define ROSE_MODULUS 			8
-#define	ROSE_MAX_PACKET_SIZE		256			/* Maximum packet size */
+#define	ROSE_MAX_PACKET_SIZE		251		/* Maximum packet size */
 
 #define	ROSE_COND_ACK_PENDING		0x01
 #define	ROSE_COND_PEER_RX_BUSY		0x02
@@ -73,6 +72,9 @@ enum {
 #define	FAC_NATIONAL_FLAGS		0x3F
 #define	FAC_NATIONAL_DEST_DIGI		0xE9
 #define	FAC_NATIONAL_SRC_DIGI		0xEB
+#define	FAC_NATIONAL_FAIL_CALL		0xED
+#define	FAC_NATIONAL_FAIL_ADD		0xEE
+#define	FAC_NATIONAL_DIGIS			0xEF
 
 #define	FAC_CCITT_DEST_NSAP		0xC9
 #define	FAC_CCITT_SRC_NSAP		0xCB
@@ -81,14 +83,17 @@ struct rose_neigh {
 	struct rose_neigh	*next;
 	ax25_address		callsign;
 	ax25_digi		*digipeat;
-	struct device		*dev;
+	ax25_cb			*ax25;
+	struct net_device		*dev;
 	unsigned short		count;
+	unsigned short		use;
 	unsigned int		number;
 	char			restarted;
 	char			dce_mode;
+	char			loopback;
 	struct sk_buff_head	queue;
-	unsigned short		t0timer, ftimer;
-	struct timer_list	timer;
+	struct timer_list	t0timer;
+	struct timer_list	ftimer;
 };
 
 struct rose_node {
@@ -96,6 +101,7 @@ struct rose_node {
 	rose_address		address;
 	unsigned short		mask;
 	unsigned char		count;
+	char			loopback;
 	struct rose_neigh	*neighbour[3];
 };
 
@@ -108,28 +114,27 @@ struct rose_route {
 	unsigned int		rand;
 };
 
-struct rose_facilities {
-	rose_address		source_addr,   dest_addr;
-	ax25_address		source_call,   dest_call;
-	unsigned char		source_ndigis, dest_ndigis;
-	ax25_address		source_digi,   dest_digi;
-	unsigned int		rand;
-};
-
 typedef struct {
 	rose_address		source_addr,   dest_addr;
 	ax25_address		source_call,   dest_call;
 	unsigned char		source_ndigis, dest_ndigis;
-	ax25_address		source_digi,   dest_digi;
+	ax25_address		source_digis[ROSE_MAX_DIGIS];
+	ax25_address		dest_digis[ROSE_MAX_DIGIS];
 	struct rose_neigh	*neighbour;
-	struct device		*device;
+	struct net_device		*device;
 	unsigned int		lci, rand;
-	unsigned char		state, condition, qbitincl;
+	unsigned char		state, condition, qbitincl, defer;
+	unsigned char		cause, diagnostic;
 	unsigned short		vs, vr, va, vl;
-	unsigned short		timer;
-	unsigned short		t1, t2, t3, hb, idle;
+	unsigned long		t1, t2, t3, hb, idle;
+#ifdef M_BIT
 	unsigned short		fraglen;
 	struct sk_buff_head	frag_queue;
+#endif
+	struct sk_buff_head	ack_queue;
+	struct rose_facilities_struct facilities;
+	struct timer_list	timer;
+	struct timer_list	idletimer;
 	struct sock		*sk;		/* Backlink to socket */
 } rose_cb;
 
@@ -151,12 +156,12 @@ extern char *rose2asc(rose_address *);
 extern struct sock *rose_find_socket(unsigned int, struct rose_neigh *);
 extern void rose_kill_by_neigh(struct rose_neigh *);
 extern unsigned int rose_new_lci(struct rose_neigh *);
-extern int  rose_rx_call_request(struct sk_buff *, struct device *, struct rose_neigh *, unsigned int);
+extern int  rose_rx_call_request(struct sk_buff *, struct net_device *, struct rose_neigh *, unsigned int);
 extern void rose_destroy_socket(struct sock *);
 
 /* rose_dev.c */
-extern int  rose_rx_ip(struct sk_buff *, struct device *);
-extern int  rose_init(struct device *);
+extern int  rose_rx_ip(struct sk_buff *, struct net_device *);
+extern int  rose_init(struct net_device *);
 
 #include <net/rosecall.h>
 
@@ -164,30 +169,43 @@ extern int  rose_init(struct device *);
 extern int  rose_process_rx_frame(struct sock *, struct sk_buff *);
 
 /* rose_link.c */
-extern void rose_link_set_timer(struct rose_neigh *);
+extern void rose_start_ftimer(struct rose_neigh *);
+extern void rose_start_t0timer(struct rose_neigh *);
+extern void rose_stop_ftimer(struct rose_neigh *);
+extern void rose_stop_t0timer(struct rose_neigh *);
+extern int  rose_ftimer_running(struct rose_neigh *);
+extern int  rose_t0timer_running(struct rose_neigh *);
 extern void rose_link_rx_restart(struct sk_buff *, struct rose_neigh *, unsigned short);
 extern void rose_transmit_restart_request(struct rose_neigh *);
 extern void rose_transmit_restart_confirmation(struct rose_neigh *);
 extern void rose_transmit_diagnostic(struct rose_neigh *, unsigned char);
-extern void rose_transmit_clear_request(struct rose_neigh *, unsigned int, unsigned char);
+extern void rose_transmit_clear_request(struct rose_neigh *, unsigned int, unsigned char, unsigned char);
 extern void rose_transmit_link(struct sk_buff *, struct rose_neigh *);
 
+/* rose_loopback.c */
+extern void rose_loopback_init(void);
+extern void rose_loopback_clear(void);
+extern int  rose_loopback_queue(struct sk_buff *, struct rose_neigh *);
+
 /* rose_out.c */
-extern void rose_output(struct sock *, struct sk_buff *);
 extern void rose_kick(struct sock *);
 extern void rose_enquiry_response(struct sock *);
-extern void rose_check_iframes_acked(struct sock *, unsigned short);
 
 /* rose_route.c */
-extern void rose_rt_device_down(struct device *);
-extern void rose_link_device_down(struct device *);
-extern struct device *rose_dev_first(void);
-extern struct device *rose_dev_get(rose_address *);
+extern struct rose_neigh *rose_loopback_neigh;
+
+extern int  rose_add_loopback_neigh(void);
+extern int  rose_add_loopback_node(rose_address *);
+extern void rose_del_loopback_node(rose_address *);
+extern void rose_rt_device_down(struct net_device *);
+extern void rose_link_device_down(struct net_device *);
+extern struct net_device *rose_dev_first(void);
+extern struct net_device *rose_dev_get(rose_address *);
 extern struct rose_route *rose_route_free_lci(unsigned int, struct rose_neigh *);
-extern struct device *rose_ax25_dev_get(char *);
-extern struct rose_neigh *rose_get_neigh(rose_address *);
+extern struct net_device *rose_ax25_dev_get(char *);
+extern struct rose_neigh *rose_get_neigh(rose_address *, unsigned char *, unsigned char *);
 extern int  rose_rt_ioctl(unsigned int, void *);
-extern void rose_link_failed(ax25_address *, struct device *);
+extern void rose_link_failed(ax25_cb *, int);
 extern int  rose_route_frame(struct sk_buff *, ax25_cb *);
 extern int  rose_nodes_get_info(char *, char **, off_t, int, int);
 extern int  rose_neigh_get_info(char *, char **, off_t, int, int);
@@ -196,14 +214,25 @@ extern void rose_rt_free(void);
 
 /* rose_subr.c */
 extern void rose_clear_queues(struct sock *);
+extern void rose_frames_acked(struct sock *, unsigned short);
+extern void rose_requeue_frames(struct sock *);
 extern int  rose_validate_nr(struct sock *, unsigned short);
 extern void rose_write_internal(struct sock *, int);
 extern int  rose_decode(struct sk_buff *, int *, int *, int *, int *, int *);
-extern int  rose_parse_facilities(struct sk_buff *, struct rose_facilities *);
+extern int  rose_parse_facilities(unsigned char *, struct rose_facilities_struct *);
 extern int  rose_create_facilities(unsigned char *, rose_cb *);
+extern void rose_disconnect(struct sock *, int, int, int);
 
 /* rose_timer.c */
-extern void rose_set_timer(struct sock *);
+extern void rose_start_heartbeat(struct sock *);
+extern void rose_start_t1timer(struct sock *);
+extern void rose_start_t2timer(struct sock *);
+extern void rose_start_t3timer(struct sock *);
+extern void rose_start_hbtimer(struct sock *);
+extern void rose_start_idletimer(struct sock *);
+extern void rose_stop_heartbeat(struct sock *);
+extern void rose_stop_timer(struct sock *);
+extern void rose_stop_idletimer(struct sock *);
 
 /* sysctl_net_rose.c */
 extern void rose_register_sysctl(void);
