@@ -17,42 +17,12 @@
 
 #include <asm/pgtable.h>
 
-/* 
- * Keep a reserved false inode which we will use to mark pages in the
- * page cache are acting as swap cache instead of file cache. 
- *
- * We only need a unique pointer to satisfy the page cache, but we'll
- * reserve an entire zeroed inode structure for the purpose just to
- * ensure that any mistaken dereferences of this structure cause a
- * kernel oops.
- */
-
-static struct inode_operations swapper_inode_operations = {
-	NULL,				/* default file operations */
-	NULL,				/* create */
-	NULL,				/* lookup */
-	NULL,				/* link */
-	NULL,				/* unlink */
-	NULL,				/* symlink */
-	NULL,				/* mkdir */
-	NULL,				/* rmdir */
-	NULL,				/* mknod */
-	NULL,				/* rename */
-	NULL,				/* readlink */
-	NULL,				/* follow_link */
-	NULL,				/* get_block */
-	NULL,				/* readpage */
-	NULL,				/* writepage */
-	block_flushpage,		/* flushpage */
-	NULL,				/* truncate */
-	NULL,				/* permission */
-	NULL,				/* smap */
-	NULL				/* revalidate */
-};
-
-struct inode swapper_inode = {
-	i_op: &swapper_inode_operations,
-	i_pages: {&swapper_inode.i_pages,&swapper_inode.i_pages}
+struct address_space swapper_space = {
+	{				/* pages	*/
+		&swapper_space.pages,	/*        .next */
+		&swapper_space.pages	/*	  .prev */
+	},
+	0				/* nrpages	*/
 };
 
 #ifdef SWAP_CACHE_INFO
@@ -70,16 +40,16 @@ void show_swap_cache_info(void)
 }
 #endif
 
-void add_to_swap_cache(struct page *page, pte_t entry)
+void add_to_swap_cache(struct page *page, swp_entry_t entry)
 {
 #ifdef SWAP_CACHE_INFO
 	swap_cache_add_total++;
 #endif
 	if (PageTestandSetSwapCache(page))
 		BUG();
-	if (page->inode)
+	if (page->mapping)
 		BUG();
-	add_to_page_cache(page, &swapper_inode, pte_val(entry));
+	add_to_page_cache(page, &swapper_space, entry.val);
 }
 
 /*
@@ -88,13 +58,14 @@ void add_to_swap_cache(struct page *page, pte_t entry)
  * Note: if swap_map[] reaches SWAP_MAP_MAX the entries are treated as
  * "permanent", but will be reclaimed by the next swapoff.
  */
-int swap_duplicate(pte_t entry)
+int swap_duplicate(swp_entry_t entry)
 {
 	struct swap_info_struct * p;
 	unsigned long offset, type;
 	int result = 0;
 
-	if (!pte_val(entry))
+	/* Swap entry 0 is illegal */
+	if (!entry.val)
 		goto out;
 	type = SWP_TYPE(entry);
 	if (type & SHM_SWP_TYPE)
@@ -115,7 +86,7 @@ int swap_duplicate(pte_t entry)
 	else {
 		static int overflow = 0;
 		if (overflow++ < 5)
-			pte_ERROR(entry);
+			printk("VM: swap entry overflow\n");
 		p->swap_map[offset] = SWAP_MAP_MAX;
 	}
 	result = 1;
@@ -123,13 +94,13 @@ out:
 	return result;
 
 bad_file:
-	pte_ERROR(entry);
+	printk("Bad swap file entry %08lx\n", entry.val);
 	goto out;
 bad_offset:
-	pte_ERROR(entry);
+	printk("Bad swap offset entry %08lx\n", entry.val);
 	goto out;
 bad_unused:
-	pte_ERROR(entry);
+	printk("Unused swap offset entry %08lx\n", entry.val);
 	goto out;
 }
 
@@ -137,10 +108,11 @@ int swap_count(struct page *page)
 {
 	struct swap_info_struct * p;
 	unsigned long offset, type;
-	pte_t entry = get_pagecache_pte(page);
+	swp_entry_t entry;
 	int retval = 0;
 
-	if (!pte_val(entry))
+	entry.val = page->index;
+	if (!entry.val)
 		goto bad_entry;
 	type = SWP_TYPE(entry);
 	if (type & SHM_SWP_TYPE)
@@ -161,23 +133,21 @@ bad_entry:
 	printk(KERN_ERR "swap_count: null entry!\n");
 	goto out;
 bad_file:
-	pte_ERROR(entry);
+	printk("Bad swap file entry %08lx\n", entry.val);
 	goto out;
 bad_offset:
-	pte_ERROR(entry);
+	printk("Bad swap offset entry %08lx\n", entry.val);
 	goto out;
 bad_unused:
-	pte_ERROR(entry);
+	printk("Unused swap offset entry %08lx\n", entry.val);
 	goto out;
 }
 
 static inline void remove_from_swap_cache(struct page *page)
 {
-	struct inode *inode = page->inode;
+	struct address_space *mapping = page->mapping;
 
-	if (!inode)
-		BUG();
-	if (inode != &swapper_inode)
+	if (mapping != &swapper_space)
 		BUG();
 	if (!PageSwapCache(page))
 		PAGE_BUG(page);
@@ -192,7 +162,9 @@ static inline void remove_from_swap_cache(struct page *page)
  */
 void __delete_from_swap_cache(struct page *page)
 {
-	pte_t entry = get_pagecache_pte(page);
+	swp_entry_t entry;
+
+	entry.val = page->index;
 
 #ifdef SWAP_CACHE_INFO
 	swap_cache_del_total++;
@@ -205,8 +177,7 @@ void __delete_from_swap_cache(struct page *page)
 
 static void delete_from_swap_cache_nolock(struct page *page)
 {
-	if (!swapper_inode.i_op->flushpage ||
-	    swapper_inode.i_op->flushpage(&swapper_inode, page, 0))
+	if (block_flushpage(NULL, page, 0))
 		lru_cache_del(page);
 
 	__delete_from_swap_cache(page);
@@ -256,7 +227,7 @@ void free_page_and_swap_cache(struct page *page)
  * lock before returning.
  */
 
-struct page * lookup_swap_cache(pte_t entry)
+struct page * lookup_swap_cache(swp_entry_t entry)
 {
 	struct page *found;
 
@@ -265,12 +236,12 @@ struct page * lookup_swap_cache(pte_t entry)
 #endif
 	while (1) {
 		/*
-		 * Right now the pagecache is 32-bit only.
+		 * Right now the pagecache is 32-bit only.  But it's a 32 bit index. =)
 		 */
-		found = find_lock_page(&swapper_inode, pte_val(entry));
+		found = find_lock_page(&swapper_space, entry.val);
 		if (!found)
 			return 0;
-		if (found->inode != &swapper_inode || !PageSwapCache(found))
+		if (found->mapping != &swapper_space || !PageSwapCache(found))
 			goto out_bad;
 #ifdef SWAP_CACHE_INFO
 		swap_cache_find_success++;
@@ -295,7 +266,7 @@ out_bad:
  * the swap entry is no longer in use.
  */
 
-struct page * read_swap_cache_async(pte_t entry, int wait)
+struct page * read_swap_cache_async(swp_entry_t entry, int wait)
 {
 	struct page *found_page = 0, *new_page;
 	unsigned long new_page_addr;
