@@ -30,6 +30,8 @@
  *			Joerg(DL1BKE)	Fixed DAMA Slave mode: will work
  *					on non-DAMA interfaces like AX25L2V2
  *					again (this behaviour is _required_).
+ *			Joerg(DL1BKE)	ax25_check_iframes_acked() returns a 
+ *					value now (for DAMA n2count handling)
  */
 
 #include <linux/config.h>
@@ -48,7 +50,7 @@
 #include <linux/inet.h>
 #include <linux/netdevice.h>
 #include <linux/skbuff.h>
-#include <linux/firewall.h>
+#include <linux/netfilter.h>
 #include <net/sock.h>
 #include <asm/uaccess.h>
 #include <asm/system.h>
@@ -56,7 +58,7 @@
 #include <linux/mm.h>
 #include <linux/interrupt.h>
 
-ax25_cb *ax25_send_frame(struct sk_buff *skb, int paclen, ax25_address *src, ax25_address *dest, ax25_digi *digi, struct device *dev)
+ax25_cb *ax25_send_frame(struct sk_buff *skb, int paclen, ax25_address *src, ax25_address *dest, ax25_digi *digi, struct net_device *dev)
 {
 	ax25_dev *ax25_dev;
 	ax25_cb *ax25;
@@ -228,6 +230,8 @@ static void ax25_send_iframe(ax25_cb *ax25, struct sk_buff *skb, int poll_bit)
 	if (skb == NULL)
 		return;
 
+	skb->nh.raw = skb->data;
+
 	if (ax25->modulus == AX25_MODULUS) {
 		frame = skb_push(skb, 1);
 
@@ -362,7 +366,7 @@ void ax25_transmit_buffer(ax25_cb *ax25, struct sk_buff *skb, int type)
 
 	ax25_addr_build(ptr, &ax25->source_addr, &ax25->dest_addr, ax25->digipeat, type, ax25->modulus);
 
-	skb->dev      = ax25->ax25_dev->dev;
+	skb->dev = ax25->ax25_dev->dev;
 
 	ax25_queue_xmit(skb);
 }
@@ -375,11 +379,6 @@ void ax25_queue_xmit(struct sk_buff *skb)
 {
 	unsigned char *ptr;
 
-	if (call_out_firewall(PF_AX25, skb->dev, skb->data, NULL, &skb) != FW_ACCEPT) {
-		kfree_skb(skb);
-		return;
-	}
-
 	skb->protocol = htons(ETH_P_AX25);
 	skb->dev      = ax25_fwd_dev(skb->dev);
 
@@ -389,20 +388,23 @@ void ax25_queue_xmit(struct sk_buff *skb)
 	dev_queue_xmit(skb);
 }
 
-void ax25_check_iframes_acked(ax25_cb *ax25, unsigned short nr)
+int ax25_check_iframes_acked(ax25_cb *ax25, unsigned short nr)
 {
 	if (ax25->vs == nr) {
 		ax25_frames_acked(ax25, nr);
 		ax25_calculate_rtt(ax25);
 		ax25_stop_t1timer(ax25);
 		ax25_start_t3timer(ax25);
+		return 1;
 	} else {
 		if (ax25->va != nr) {
 			ax25_frames_acked(ax25, nr);
 			ax25_calculate_t1(ax25);
 			ax25_start_t1timer(ax25);
+			return 1;
 		}
 	}
+	return 0;
 }
 
 #endif
